require "chickadee/container"
require "rack/request"
require "rack/response"
require "rack/builder"

module Chickadee
  class Application
    def initialize(request: Rack::Request, response: Rack::Response, builder: Rack::Builder, container: Container)
      @request_class = request
      @response_class = response
      @container_class = container
      @builder_class = builder
      @use_cache = []
      @register_cache = []
    end

    def register(*args, &block)
      @register_cache << [args, block]
    end

    def use(*args, &block)
      @use_cache << [args, block]
    end

    def service(name)
      builder = @builder_class.new
      @use_cache.each { |use| builder.use(*use[0], &use[1]) }
      builder.run(service_app(name))
      builder
    end

    private

    def service_app(name)
      Proc.new do |env|
        request = @request_class.new(env)
        response = @response_class.new
        container = @container_class.new
        @register_cache.each{ |r| container.register(*r[0], &r[1]) }
        container.resolve(name).call(request, response)
      end
    end
  end
end